#Copyright 2009 Diego Duclos
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
import gtk
from model import marketGroup, graphic, category, group, item
from gui import pixbufLoader
import copy
import re

def init(self):
    #Set precedent group variable to nothin'
    self.precedentGroup = None
    #Load favorites
    if not "favoriteItems" in self.guiSettings: self.guiSettings["favoriteItems"] = set()
    self.favoriteItems = self.guiSettings["favoriteItems"]
    #Check if favoriteItems is still in the old format and if so, convert
    newFavs = set()
    for i in self.favoriteItems:
        if isinstance(i, item.item):
            newFavs.add(i.ID)
    if len(newFavs) > 0: self.favoriteItems = newFavs
            
    if not "recentItems" in self.guiSettings: self.guiSettings["recentItems"] = []
    self.recentItems = self.guiSettings["recentItems"]
    for i in range(len(self.recentItems)):
        if isinstance(self.recentItems[i], item.item):
            self.recentItems[i] = self.recentItems[i].ID
    
    store = gtk.TreeStore(gtk.gdk.Pixbuf, str, object)
    store.set_sort_column_id(1, gtk.SORT_ASCENDING)
    store.set_sort_func(1, self.groupSort)
    self.tvwGroups.set_model(store)
    self.tvwGroups.set_tooltip_column(2)
    self.tvwGroups.set_headers_visible(False)
    
    imageRenderer = gtk.CellRendererPixbuf()
    nameRenderer = gtk.CellRendererText()
    col = gtk.TreeViewColumn()

    col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
    col.pack_start(imageRenderer, False)
    col.pack_start(nameRenderer, True)
    col.add_attribute(imageRenderer, 'pixbuf', 0)
    col.add_attribute(nameRenderer, 'text', 1)
    self.tvwGroups.append_column(col)
    #Modules
    mGroupMods = marketGroup.getMarketGroup(name = "Ship Equipment")
    self.buildGroups(mGroupMods, None, True)
    #Rigs
    mGroupRigs = marketGroup.getMarketGroup(name = "Rigs")
    self.buildGroups(mGroupRigs, None, True)
    #Drones
    mGroupDrones = marketGroup.getMarketGroup(name = "Drones")
    self.buildGroups(mGroupDrones, None, True)
    #Ammo
    mGroupAmmo = marketGroup.getMarketGroup(name = "Ammunition & Charges")
    self.buildGroups(mGroupAmmo, None, True)
    #Subsystems. Can't look by market group name here or there's a chance we'll get the skills
    mGroupSubsystems = marketGroup.getMarketGroup(ID = 1112)
    self.buildGroups(mGroupSubsystems, None, True)
    #Implants & Boosters
    mGroupImplantsBoosters = marketGroup.getMarketGroup(name = "Implants & Boosters")
    iter = self.addGroup(mGroupImplantsBoosters.name, mGroupImplantsBoosters, None)
    cat = category.getCategory(name = "Implant")
    grps = group.getGroupsByCategory(cat.ID)
    for grp in grps:
        if grp.published == 1:
            if grp.name == "Booster":
                pixbuf = pixbufLoader.getPixbuf("booster")
            else:
                pixbuf = pixbufLoader.getPixbuf("hardwiring")
            store.append(iter, (pixbuf, grp.name, grp))
    
    #Wormhole effect beacons
    groupWH = group.getGroup(ID = 920)
    iter = store.append(None, (None, "Wormhole Effects", None))
    #We need to classify the wormhole effects into groups, we'll use a regex match for that
    groups = set()
    items = item.getItemsByGroup(group = groupWH.ID)
    effectBeaconRe = re.compile("(.*) Effect Beacon Class (.*)")
    for i in items:
        if re.search(effectBeaconRe, i.name):
            t, c = re.match(effectBeaconRe, i.name).groups()
            groups.add(t)
    
    #Add Red Giants manualy because their names are different for some reason
    groups.add("Red Giant")
    
    for g in groups: store.append(iter, (None, g, "wh"))
    
    #Favorites
    store.append(None, (None, "Favorites", "favorite"))
    #Recently used
    store.append(None, (None, "Recently Used", "recent"))
    
    #Connect events
    self.tvwGroups.connect("cursor-changed", self.groupSelected)
    self.tvwGroups.connect("button-press-event", self.toggleGroup)

def getSelectedGroup(self):
    store = self.tvwGroups.get_model()
    path = self.tvwGroups.get_cursor()[0]
    if path == None or path == -1: return None
    iter = store.get_iter(path)
    return store.get_value(iter, 2)

def groupSort(self, store, iter1, iter2):
    #Put stuff of type group or marketGroup at top, after then the rest
    grp1, grp2 = store.get_value(iter1, 2), store.get_value(iter2, 2)
    prio1 = isinstance(grp1, marketGroup.marketGroup) or isinstance(grp1, group.group) or grp1 == None
    prio2 = isinstance(grp2, marketGroup.marketGroup) or isinstance(grp2, group.group) or grp2 == None
    if prio1 != prio2: return prio1 and -1 or 1
    else:
        name1, name2 = store.get_value(iter1, 1), store.get_value(iter2, 1)
        return cmp(name1, name2)
        

def toggleGroup(self, widget, event):
    widget.do_button_press_event(widget, event)
    if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
        path = self.tvwGroups.get_cursor()[0]
        if path != None and path != -1:
            if self.tvwGroups.row_expanded(path):
                self.tvwGroups.collapse_row(path)
            else: self.tvwGroups.expand_row(path, False)
        
def expandToItem(self, itemList):
    if isinstance(itemList, list) or isinstance(itemList, tuple): itm = itemList[0]
    else: itm = itemList
    if not itm.marketGroupID: baseItem = item.getBase(itm.ID) or itm
    else: baseItem = itm
    path = self.lookForGroup(baseItem.marketGroup) or self.lookForGroup(baseItem.group)
    self.tvwGroups.expand_to_path(path)
    self.tvwGroups.set_cursor(path)
    itemTabs = []
    if isinstance(itemList, list) or isinstance(itemList, tuple):
        for itm in itemList:
            metaGroup = itm.metaGroupID
            if metaGroup == 4 and not self.tbnFaction in itemTabs: itemTabs.append(self.tbnFaction)
            elif metaGroup == 5 and not self.tbnOfficer in itemTabs: itemTabs.append(self.tbnOfficer)
            elif metaGroup == 6 and not self.tbnComplex in itemTabs: itemTabs.append(self.tbnComplex)
            elif metaGroup not in (4, 5, 6) and not self.tbnNormal in itemTabs: itemTabs.append(self.tbnNormal)
    else:
        metaGroup = itm.metaGroupID
        if metaGroup == 4: itemTabs.append(self.tbnFaction)
        elif metaGroup == 5: itemTabs.append(self.tbnOfficer)
        elif metaGroup == 6: itemTabs.append(self.tbnComplex)
        else: itemTabs.append(self.tbnNormal)
    for candidate in (self.tbnNormal, self.tbnFaction, self.tbnComplex, self.tbnOfficer):
        if candidate in itemTabs: candidate.set_active(True)
        else: candidate.set_active(False)

def lookForGroup(self, neededGroup, iter = None, store = None, i = 0):
    store = store or self.tvwGroups.get_model()
    iter = iter or store.get_iter((0,))
    while iter != None:
        group = store.get_value(iter, 2)
        if group == neededGroup: return store.get_path(iter)
        iterChild = store.iter_children(iter)
        if iterChild != None:
            inGroup = self.lookForGroup(neededGroup, iterChild, store, i + 1)
            if inGroup != None:
                return inGroup
        
        iter = store.iter_next(iter)

def buildGroups(self, parentGroup, parentIter, includeParent = False):
    store  = self.tvwGroups.get_model()
    if includeParent:
        parentIter = self.addGroup(parentGroup.name, parentGroup, parentIter)
    
    if parentGroup.children:
        for name, mGroup in parentGroup.children.iteritems():
            if name != "Deployable Equipment":
                iter = self.addGroup(name, mGroup, parentIter)
                self.buildGroups(mGroup, iter, False)
        
    if includeParent: return parentIter

def groupSelected(self, *stuff):    
    store = self.tvwGroups.get_model()
    path = self.tvwGroups.get_cursor()[0]
    if path == None or path == -1: return
    iter = store.get_iter(path)
    name, grp = store.get(iter, 1, 2)

    #Normal items, those are in market groups so it's easy
    store = self.tvwItems.get_model()
    store.clear()
    dovars = False
    if isinstance(grp, marketGroup.marketGroup):
        dovars = True
        items = item.getItemsByGroup(grp)     
        if self.itemBrowserSlotCol in self.tvwItems.get_columns():
            self.tvwItems.remove_column(self.itemBrowserSlotCol)
    elif isinstance(grp, group.group):
        dovars = True
        items = item.getItemsByGroup(group = grp)
        if not self.itemBrowserSlotCol in self.tvwItems.get_columns():
            self.tvwItems.insert_column(self.itemBrowserSlotCol, 0)
    elif grp == "recent" or grp == "favorite":
        if grp != self.precedentGroup:
            for tbn in (self.tbnFaction, self.tbnComplex, self.tbnOfficer):
                tbn.set_active(True)
        if grp == "recent":
            items = copy.copy(self.recentItems)
            for i in range(len(items)): items[i] = item.getItem(ID = items[i])
        elif grp == "favorite":
            items = []
            for i in self.favoriteItems: items.append(item.getItem(ID = i))
    elif grp == "wh":
        items = []
        effectBeaconRe = re.compile("%s Effect Beacon Class (.*)" % name)
        effectBeaconRe2 = re.compile("%s Beacon Class (.*)" % name)
        beacons = item.getItemsByGroup(group = 920)
        for i in beacons:
            if re.search(effectBeaconRe, i.name) or re.search(effectBeaconRe2, i.name):
                items.append(i)
    else: items = []
    
    if ((self.precedentGroup == "favorite" or self.precedentGroup == "recent") or self.itemSearchActive) and grp != "favorite" and grp != "recent":
        self.itemSearchActive = False
        self.precedentGroup = grp
        self.tbnNormal.set_active(True)
        for tbn in (self.tbnFaction, self.tbnComplex, self.tbnOfficer):
            tbn.set_active(False)
        return
    
    if self.itemSearchActive:
        self.itemSearchActive = False
        return
    
    self.precedentGroup = grp

    metaIds = set()
    for i in items:
        if i and i.published == 1:
            #append the item itself
            pixbuf = pixbufLoader.getPixbuf(i.icon, True, "16x16")
            if i.group.category.name == "Implant":
                if i.group.name == "Booster":
                    slot = "%d" % i.getModifiedAttribute("boosterness")
                else:
                    slot = "%d" % i.getModifiedAttribute("implantness")
            else:
                slot = None
            if self.itemFilter(i):
                store.append((pixbuf, i.name, i.description, i, slot))
            
            metaIds.add(i.metaGroupID)
            #handle variations
            if dovars:
                for v in item.getVariations(i):
                    if v not in items and v.metaGroupID != 2:
                        metaIds.add(v.metaGroupID)
                        if self.itemFilter(v): store.append((pixbuf, v.name, v.description, v, slot))
    
    if 4 in metaIds: self.tbnFaction.set_sensitive(True)
    else: self.tbnFaction.set_sensitive(False)
    if 5 in metaIds: self.tbnOfficer.set_sensitive(True)
    else: self.tbnOfficer.set_sensitive(False)
    if 6 in metaIds: self.tbnComplex.set_sensitive(True)
    else: self.tbnComplex.set_sensitive(False)
    
    fallback = True
    for metaId in (4, 5, 6):
        if metaId in metaIds: fallback = False
        
    if fallback and not self.tbnNormal.get_active(): self.tbnNormal.set_active(True)
    
def addGroup(self, name, mGroup, iter):
    store = self.tvwGroups.get_model()
    if mGroup.graphicID != None:
        icon = graphic.getIconName(mGroup.graphicID)
        pixbuf = pixbufLoader.getPixbuf(icon, True, "16x16")
    else:
        pixbuf = None
            
    return store.append(iter, (pixbuf, name, mGroup))
